import os
from pylab import *
rc('font', size=12)

amgcl_amg = loadtxt('dmem_data/lrz/ns_amgcl_amg.txt', dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })
amgcl_sdd = loadtxt('dmem_data/lrz/ns_amgcl_sdd.txt', dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })
trilinos = loadtxt('dmem_data/lrz/ns_trilinos.txt', dtype={
            'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
            'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
            })

omp_amg = unique(list(amgcl_amg['omp']))
omp_sdd = unique(list(amgcl_sdd['omp']))

def set_ticks(ax):
    gca().set_xscale('log')
    gca().set_xticks([28, 28 * 4, 28 * 16, 28 * 64])
    gca().get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    gca().get_xaxis().set_tick_params(which='minor', size=0)
    gca().get_xaxis().set_tick_params(which='minor', width=0)

fig,gs = subplots(2, 2, figsize=(8,7))
handles = []

ax = gs[0,0]
for n in omp_amg:
    I = (amgcl_amg['omp'] == n)
    h = ax.loglog(n * amgcl_amg[I]['mpi'], amgcl_amg[I]['setup'] + amgcl_amg[I]['solve'], 'o-')
    handles.append(h[0])
for n in omp_sdd:
    I = (amgcl_sdd['omp'] == n)
    h = ax.loglog(n * amgcl_sdd[I]['mpi'], amgcl_sdd[I]['setup'] + amgcl_sdd[I]['solve'], 'v:')
    handles.append(h[0])
h = ax.loglog(trilinos['mpi'], trilinos['setup'] + trilinos['solve'], 's-')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Total time')

ax = gs[0,1]
for n in omp_amg:
    I = (amgcl_amg['omp'] == n)
    h = ax.loglog(n * amgcl_amg[I]['mpi'], amgcl_amg[I]['setup'], 'o-')
    handles.append(h[0])
for n in omp_sdd:
    I = (amgcl_sdd['omp'] == n)
    h = ax.loglog(n * amgcl_sdd[I]['mpi'], amgcl_sdd[I]['setup'], 'v:')
    handles.append(h[0])
h = ax.loglog(trilinos['mpi'], trilinos['setup'], 's-')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Setup time')

ax = gs[1,0]
for n in omp_amg:
    I = (amgcl_amg['omp'] == n)
    h = ax.loglog(n * amgcl_amg[I]['mpi'], amgcl_amg[I]['solve'], 'o-')
    handles.append(h[0])
for n in omp_sdd:
    I = (amgcl_sdd['omp'] == n)
    h = ax.loglog(n * amgcl_sdd[I]['mpi'], amgcl_sdd[I]['solve'], 'v:')
    handles.append(h[0])
h = ax.loglog(trilinos['mpi'], trilinos['solve'], 's-')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Solve time')

ax = gs[1,1]
for n in omp_amg:
    I = (amgcl_amg['omp'] == n)
    h = ax.semilogx(n * amgcl_amg[I]['mpi'], amgcl_amg[I]['iters'], 'o-')
    handles.append(h[0])
for n in omp_sdd:
    I = (amgcl_sdd['omp'] == n)
    h = ax.semilogx(n * amgcl_sdd[I]['mpi'], amgcl_sdd[I]['iters'], 'v:')
    handles.append(h[0])
h = ax.semilogx(trilinos['mpi'], trilinos['iters'], 's-')
handles.append(h[0])
set_ticks(ax)
ax.set_ylabel('Iterations')

fig.tight_layout()

figlegend(handles,
    ['AMG, omp={}'.format(n) for n in omp_amg] +
    ['SDD, omp={}'.format(n) for n in omp_sdd] +
    ['Trilinos'],
    ncol=3, loc='lower center')
fig.suptitle('Strong scaling of the Navier-Stokes problem on the SuperMUC cluster')
fig.subplots_adjust(top=0.93, bottom=0.2)

show()